@@ -466,17 +466,17 @@ DataImpl :: load_headers (const DataIO   & data_io,
         a.message_id = s;
 
         // subject line
-        in->getline (s); s.trim(); a.subject = s;
+        in->getline (s); s.ltrim(); a.subject = s;
 
         // author line
-        in->getline (s); s.trim(); a.author = s.len==1 ? author_lookup[(int)*s.str] : Quark(s);
+        in->getline (s); s.ltrim(); a.author = s.len==1 ? author_lookup[(int)*s.str] : Quark(s);
 
         // optional references line
         std::string references;
-        in->getline (s); s.trim();
+        in->getline (s); s.ltrim();
         if (!s.empty() && *s.str=='<') {
           references = s;
-          in->getline (s); s.trim();
+          in->getline (s); s.ltrim();
         }
 
         // date-posted line
@@ -484,7 +484,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
         const int days_old ((now - a.time_posted) / (24*60*60));
 
         // xref line
-        in->getline (s); s.trim();
+        in->getline (s); s.ltrim();
         const size_t max_targets (std::count (s.begin(), s.end(), ' ') + 1);
         targets_v.resize (max_targets);
         Xref::Target * target_it (&targets_v.front());
@@ -508,11 +508,11 @@ DataImpl :: load_headers (const DataIO   & data_io,
         int total_part_count (1);
         int found_part_count (1);
         in->getline (s);
-        s.trim();
-        s.pop_token (tok); s.trim(); a.is_binary = !tok.empty() && tok.str[0]=='t';
+        s.ltrim();
+        s.pop_token (tok); a.is_binary = !tok.empty() && tok.str[0]=='t';
         if (a.is_binary) {
-          s.pop_token (tok); total_part_count = atoi(tok.str);
-          s.pop_token (tok); found_part_count = atoi(tok.str);
+          s.ltrim(); s.pop_token (tok); total_part_count = atoi(tok.str);
+          s.ltrim(); s.pop_token (tok); found_part_count = atoi(tok.str);
         }
         if (!expired)
           a.set_part_count (total_part_count);
@@ -520,14 +520,15 @@ DataImpl :: load_headers (const DataIO   & data_io,
         // found parts...
         for (int i(0), count(found_part_count); i<count; ++i)
         {
-          in->getline (s);
+          const bool gotline (in->getline (s));
 
-          if (!expired)
+          if (gotline && !expired)
           {
-            Article::Part& p (a.get_part (i+1));
             StringView tok;
             s.trim ();
-            s.pop_token (tok); tok.trim(); p.index = view_to_ul (tok);
+            s.pop_token (tok); tok.trim(); const unsigned long index (view_to_ul (tok));
+            Article::Part& p (a.get_part (index));
+            p.index = index;
             s.pop_token (tok); tok.trim(); if (tok.len==1 && *tok.str=='"') p.message_id = a.message_id.c_str(); else p.message_id.assign (tok.str, tok.len);
             s.pop_token (tok); tok.trim(); p.bytes = view_to_ul (tok);
             s.pop_token (tok); tok.trim(); p.lines = view_to_ul (tok);
