@@ -188,14 +188,14 @@ void
 ArticleCache :: fire_added (const Quark& mid)
 {
   for (listeners_t::iterator it(_listeners.begin()), end(_listeners.end()); it!=end; )
-    (*it++)->onCacheAdded (mid);
+    (*it++)->on_cache_added (mid);
 }
 
 void
 ArticleCache :: fire_removed (const quarks_t& mids)
 {
   for (listeners_t::iterator it(_listeners.begin()), end(_listeners.end()); it!=end; )
-    (*it++)->onCacheRemoved (mids);
+    (*it++)->on_cache_removed (mids);
 }
 
 /*****
