@@ -114,7 +114,7 @@ namespace
   void do_refresh_groups               (GtkAction * a) { pan_ui->do_refresh_groups(); }
   void do_subscribe_selected_groups    (GtkAction * a) { pan_ui->do_subscribe_selected_groups(); }
   void do_unsubscribe_selected_groups  (GtkAction * a) { pan_ui->do_unsubscribe_selected_groups(); }
-  void do_save_attachments             (GtkAction * a) { pan_ui->do_save_attachments(); }
+  void do_save_articles                (GtkAction * a) { pan_ui->do_save_articles(); }
   void do_print                        (GtkAction * a) { pan_ui->do_print(); }
   void do_import_tasks                 (GtkAction * a) { pan_ui->do_import_tasks(); }
   void do_cancel_latest_task           (GtkAction * a) { pan_ui->do_cancel_latest_task(); }
@@ -162,7 +162,8 @@ namespace
   void do_mark_article_read            (GtkAction * a) { pan_ui->do_mark_article_read(); }
   void do_mark_article_unread          (GtkAction * a) { pan_ui->do_mark_article_unread(); }
   void do_post                         (GtkAction * a) { pan_ui->do_post(); }
-  void do_followup                     (GtkAction * a) { pan_ui->do_followup(); }
+  void do_followup_to                  (GtkAction * a) { pan_ui->do_followup_to(); }
+  void do_reply_to                     (GtkAction * a) { pan_ui->do_reply_to(); }
   void do_pan_web                      (GtkAction * a) { pan_ui->do_pan_web(); }
   void do_bug_report                   (GtkAction * a) { pan_ui->do_bug_report(); }
   void do_tip_jar                      (GtkAction * a) { pan_ui->do_tip_jar(); }
@@ -267,7 +268,7 @@ namespace
     { "actions-menu", NULL, N_("_Actions") },
     { "article-actions-menu", NULL, N_("_Articles") },
     { "group-actions-menu", NULL, N_("_Groups") },
-    { "posting-actions-menu", NULL, N_("_Posting") },
+    { "posting-actions-menu", NULL, N_("_Post") },
     { "post-menu", NULL, N_("_Post") },
     { "help-menu", NULL, N_("_Help") },
     { "read-selected-group", "ICON_READ_MORE", N_("_Read Group"), NULL, N_("Read Selected Group"), G_CALLBACK(do_read_selected_group) },
@@ -280,7 +281,7 @@ namespace
     { "subscribe", NULL, N_("_Subscribe"), NULL, N_("Subscribe to Selected Groups"), G_CALLBACK(do_subscribe_selected_groups) },
     { "unsubscribe", NULL, N_("_Unsubscribe"), NULL, N_("Unsubscribe from Selected Groups"), G_CALLBACK(do_unsubscribe_selected_groups) },
 
-    { "save-attachments", GTK_STOCK_SAVE, N_("_Save Attachments..."), "<shift>S", N_("Save Selected Articles' Attachments..."), G_CALLBACK(do_save_attachments) },
+    { "save-articles", GTK_STOCK_SAVE, N_("_Save Articles..."), "<shift>S", N_("Save Selected Articles..."), G_CALLBACK(do_save_articles) },
     { "print", GTK_STOCK_PRINT, NULL, "<control>P", NULL, G_CALLBACK(do_print) },
     { "import-tasks", GTK_STOCK_OPEN, N_("_Import NZB Files..."), NULL, NULL, G_CALLBACK(do_import_tasks) },
     { "cancel-last-task", GTK_STOCK_CANCEL, N_("_Cancel Last Task"), "<control>Delete", N_("Cancel Last Task"), G_CALLBACK(do_cancel_latest_task) },
@@ -301,7 +302,7 @@ namespace
 
     { "jump-to-group-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Group Tab"), "1", NULL, G_CALLBACK(do_jump_to_group_tab) },
     { "jump-to-header-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Header Tab"), "2", NULL, G_CALLBACK(do_jump_to_header_tab) },
-    { "jump-to-body-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Header Tab"), "3", NULL, G_CALLBACK(do_jump_to_body_tab) },
+    { "jump-to-body-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Body Tab"), "3", NULL, G_CALLBACK(do_jump_to_body_tab) },
     { "rot13-selected-text", NULL, N_("_Rot13 Selected Text"), "<control><shift>R", NULL, G_CALLBACK(do_rot13_selected_text) },
 
     { "clear-header-pane", NULL, N_("Clear _Header Pane"), NULL, N_("Clear Header Pane"), G_CALLBACK(do_clear_header_pane) },
@@ -333,8 +334,9 @@ namespace
     { "mark-article-read", "ICON_ARTICLE_READ", N_("_Mark Article as Read"), "M", N_("Mark Article as Read"), G_CALLBACK(do_mark_article_read) },
     { "mark-article-unread", "ICON_ARTICLE_UNREAD", N_("Mark Article as _Unread"), "<control>M", N_("Mark Article as Unread"), G_CALLBACK(do_mark_article_unread) },
 
-    { "post", "ICON_COMPOSE_POST", N_("_Post"), "P", N_("Post New Article"), G_CALLBACK(do_post) },
-    { "followup", "ICON_COMPOSE_FOLLOWUP", N_("_Follow-up"), "F", N_("Follow-up to Article"), G_CALLBACK(do_followup) },
+    { "post", "ICON_COMPOSE_POST", N_("_Post to Newsgroup"), "P", N_("Post to Newsgroup"), G_CALLBACK(do_post) },
+    { "followup-to", "ICON_COMPOSE_FOLLOWUP", N_("_Followup to  Newsgroup"), "F", N_("Followup to Newsgroup"), G_CALLBACK(do_followup_to) },
+    { "reply-to", "ICON_COMPOSE_FOLLOWUP", N_("_Reply to Author"), "R", N_("Reply to Author"), G_CALLBACK(do_reply_to) },
 
     { "pan-web-page", NULL, N_("_Pan Home Page..."), NULL, NULL, G_CALLBACK(do_pan_web) },
     { "bug-report", NULL, N_("Give _Feedback or Report a Bug..."), NULL, NULL, G_CALLBACK(do_bug_report) },
@@ -394,13 +396,19 @@ namespace
   {
     SHOW_ARTICLES,
     SHOW_THREADS,
-    SHOW_SUBTHREADS
+    SHOW_SUBTHREADS,
+    SHOW_N
   };
 
+  static const char * show_strings[SHOW_N] = { "articles", "threads", "subthreads" };
+
   void
   show_matches_cb (GtkAction *action, GtkRadioAction *current)
   {
-    pan_ui->do_show_matches (Data::ShowType(gtk_radio_action_get_current_value (current)));
+    const int mode (gtk_radio_action_get_current_value (current));
+    g_assert (0<=mode && mode<SHOW_N);
+    prefs->set_string ("header-pane-show-matching", show_strings[mode]);
+    pan_ui->do_show_matches (Data::ShowType(mode));
   }
 
   GtkRadioActionEntry match_toggle_entries[] =
@@ -470,6 +478,14 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
     }
   }
 
+  const std::string show_str (p->get_string ("header-pane-show-matching", "articles"));
+  int show_mode (SHOW_ARTICLES);
+  for (int i=0; i<SHOW_N; ++i) {
+    if (show_str == show_strings[i]) {
+      show_mode = i;
+      break;
+    }
+  }
 
   GtkActionGroup * action_group = _group = gtk_action_group_new ("Actions");
   gtk_action_group_set_translation_domain (action_group, NULL);
@@ -477,7 +493,7 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
   gtk_action_group_add_toggle_actions (action_group, toggle_entries, n_toggle_entries, NULL);
   gtk_action_group_add_radio_actions (action_group,
                                       match_toggle_entries, G_N_ELEMENTS(match_toggle_entries),
-                                      SHOW_ARTICLES,
+                                      show_mode,
                                       G_CALLBACK(show_matches_cb), NULL);
   gtk_action_group_add_radio_actions (action_group,
                                       fallback_charsets, G_N_ELEMENTS(fallback_charsets),
