@@ -52,7 +52,8 @@ namespace
   class PixbufCache
   {
     private:
-      typedef std::map<int,GdkPixbuf*> pixbufs_t;
+      typedef std::pair<GdkPixbuf*,GdkPixbuf*> pixbuf_pair_t; // original, scaled
+      typedef std::map<int,pixbuf_pair_t> pixbufs_t;
       pixbufs_t _pixbufs;
 
     public:
@@ -61,24 +62,31 @@ namespace
 
     public:
       void clear () {
-        foreach (pixbufs_t, _pixbufs, it)
-          g_object_unref (it->second);
+        foreach (pixbufs_t, _pixbufs, it) {
+          g_object_unref (it->second.first);
+          g_object_unref (it->second.second);
+        }
         _pixbufs.clear ();
       }
 
     public:
-      GdkPixbuf* get_pixbuf_at_offset (int offset) {
-        GdkPixbuf* ret (0);
-        if (_pixbufs.count(offset))
-          ret = _pixbufs[offset];
-        return ret;
+      bool get_pixbuf_at_offset (int offset, GdkPixbuf*& original, GdkPixbuf*& scaled) {
+        if (!_pixbufs.count(offset))
+          return false;
+        pixbuf_pair_t &p(_pixbufs[offset]);
+        original = p.first;
+        scaled = p.second;
+        return true;
       }
-      void set_pixbuf_at_offset (int offset, GdkPixbuf* new_p) {
-        GdkPixbuf *& p (_pixbufs[offset]);
-        if (p)
-          g_object_unref (p);
-        p = new_p;
-        g_object_ref (p);
+      void set_pixbuf_at_offset (int offset, GdkPixbuf *original, GdkPixbuf *scaled) {
+        const bool had_old (_pixbufs.count(offset));
+        pixbuf_pair_t &p(_pixbufs[offset]), old(p);
+        g_object_ref (p.first = original);
+        g_object_ref (p.second = scaled);
+        if (had_old) {
+          g_object_unref (old.first);
+          g_object_unref (old.second);
+        }
       }
   };
 
@@ -97,12 +105,12 @@ namespace
     return *pc;
   }
 
-  GdkPixbuf* get_pixbuf_at_offset (gpointer o, int offset) {
-    return get_pixbuf_cache(o).get_pixbuf_at_offset (offset);
+  bool get_pixbuf_at_offset (gpointer o, int offset, GdkPixbuf*& original, GdkPixbuf*& scaled) {
+    return get_pixbuf_cache(o).get_pixbuf_at_offset (offset, original, scaled);
   }
 
-  void set_pixbuf_at_offset (gpointer o, int offset, GdkPixbuf* p) {
-    get_pixbuf_cache(o).set_pixbuf_at_offset (offset, p);
+  void set_pixbuf_at_offset (gpointer o, int offset, GdkPixbuf *original, GdkPixbuf *scaled) {
+    get_pixbuf_cache(o).set_pixbuf_at_offset (offset, original, scaled);
   }
 
   void clear_pixbuf_cache (gpointer o) {
@@ -174,6 +182,14 @@ namespace
     }
   }
 
+  void set_cursor (GdkWindow *window, GtkWidget *w, int mode)
+  {
+    ensure_cursors_created (w);
+    GdkCursor * cursor_new = cursors[mode];
+    if (cursor_new != cursor_current)
+      gdk_window_set_cursor (window, cursor_current=cursor_new);
+  }
+
   void set_cursor_from_iter (GdkWindow *window, GtkWidget *w, GtkTextIter *it)
   {
     GtkTextView * text_view (GTK_TEXT_VIEW (w));
@@ -185,21 +201,12 @@ namespace
     const bool in_pix (gtk_text_iter_has_tag (it, pix_tag));
     const bool fullsize (get_fullsize_flag (buf));
 
-    ensure_cursors_created (w);
-
-    GdkCursor * cursor_new;
-    if (in_pix && fullsize)
-      cursor_new = cursors[CURSOR_ZOOM_OUT];
-    else if (in_pix)
-      cursor_new = cursors[CURSOR_ZOOM_IN];
-    else if (in_url)
-      cursor_new = cursors[CURSOR_HREF];
-    else
-      cursor_new = cursors[CURSOR_IBEAM];
-
-    // change the cursor if needed
-    if (cursor_new != cursor_current)
-      gdk_window_set_cursor (window, cursor_current=cursor_new);
+    int mode;
+    if (in_pix && fullsize) mode = CURSOR_ZOOM_OUT;
+    else if (in_pix)        mode = CURSOR_ZOOM_IN;
+    else if (in_url)        mode = CURSOR_HREF;
+    else                    mode = CURSOR_IBEAM;
+    set_cursor (window, w, mode);
   }
 }
 
@@ -223,7 +230,6 @@ namespace
                                            GTK_TEXT_WINDOW_WIDGET,
                                            x, y, &x, &y);
     gtk_text_view_get_iter_at_location (text_view, setme, x, y);
-    gtk_text_iter_backward_char (setme);
   }
 
   gboolean motion_notify_event (GtkWidget       * w,
@@ -249,12 +255,11 @@ namespace
 
   char* get_url_from_location (GtkWidget * w, int x, int y)
   {
-    GtkTextIter iter;
-    get_iter_from_event_coords (w, x, y, &iter);
-
-    GtkTextTag * url_tag (get_named_tag_from_view (w, "url"));
-
     char * retval (0);
+
+    GtkTextIter iter;
+    GtkTextTag * url_tag (get_named_tag_from_view (w, "url"));
+    get_iter_from_event_coords (w, x, y, &iter);
     if (gtk_text_iter_has_tag (&iter, url_tag))
     {
       GtkTextIter begin(iter), end(iter);
@@ -303,55 +308,69 @@ namespace
                                const GtkAllocation  * size,
                                GtkTextTag           * apply_tag)
   {
-    const int offset (gtk_text_iter_get_offset (iter));
+    const int begin_offset (gtk_text_iter_get_offset (iter));
 
-    GdkPixbuf * original (get_pixbuf_at_offset (buf, offset));
+    GdkPixbuf * original (0);    
+    GdkPixbuf * old_scaled (0);    
+    if (!get_pixbuf_at_offset (buf, begin_offset, original, old_scaled))
+      return;
 
-    // remove the old
-    GtkTextIter end (*iter);
-    gtk_text_iter_forward_to_tag_toggle (&end, apply_tag);
-    gtk_text_buffer_delete (buf, iter, &end);
+    GdkPixbuf * new_scaled (size_to_fit (original, (fullsize ? 0 : size)));
+    const int old_w (gdk_pixbuf_get_width (old_scaled));
+    const int new_w (gdk_pixbuf_get_width (new_scaled));
+    const int old_h (gdk_pixbuf_get_height (old_scaled));
+    const int new_h (gdk_pixbuf_get_height (new_scaled));
+    if (old_w!=new_w || old_h!=new_h)
+    {
+      // remove the old..
+      GtkTextIter old_end (*iter);
+      gtk_text_iter_forward_to_tag_toggle (&old_end, apply_tag);
+      gtk_text_buffer_delete (buf, iter, &old_end);
 
-    // insert the new
-    GdkPixbuf * scaled (size_to_fit (original, (fullsize ? 0 : size)));
-    gtk_text_buffer_insert_pixbuf (buf, iter, scaled);
-    g_object_unref (scaled);
+      // insert the new..
+      gtk_text_buffer_insert_pixbuf (buf, iter, new_scaled);
+      gtk_text_buffer_insert (buf, iter, "\n", -1);
+      set_pixbuf_at_offset (buf, begin_offset, original, new_scaled);
 
-    // and apply the tag.
-    end = *iter;
-    gtk_text_iter_set_offset (iter, offset);
-    gtk_text_buffer_apply_tag (buf, apply_tag, iter, &end);
-    *iter = end;
+      // and apply the tag.
+      GtkTextIter begin (*iter);
+      gtk_text_iter_set_offset (&begin, begin_offset);
+      gtk_text_buffer_apply_tag (buf, apply_tag, &begin, iter);
+    }
+
+    g_object_unref (new_scaled);
   }
 
   gboolean mouse_button_pressed (GtkWidget       * w,
                                  GdkEventButton  * event,
-                                 gpointer          unused)
+                                 gpointer          prefs)
   {
     g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), false);
 
     if (event->button==1 || event->button==2) {
       char * url (get_url_from_location (w, (int)event->x, (int)event->y));
       if (url) {
-        /* this is kind of a crude way of making sure that double-click
+        /* this is a crude way of making sure that double-click
          * doesn't open two or three browser windows. */
         static time_t last_url_time (0);
-        const time_t this_url_time (time (NULL));
+        const time_t this_url_time (time (0));
         if (this_url_time != last_url_time) {
           last_url_time = this_url_time;
-          pan_url_show (url);
+          URL :: open (*static_cast<Prefs*>(prefs), url);
           g_free (url);
         }
       } else { // maybe we're zooming in/out on a pic...
         GtkTextIter iter;
+        GtkTextTag * pix_tag (get_named_tag_from_view (w, "pixbuf"));
         get_iter_from_event_coords (w, (int)event->x, (int)event->y, &iter);
-        GtkTextTag * tag (get_named_tag_from_view (w, "pixbuf"));
-        if (gtk_text_iter_begins_tag (&iter, tag))
+        if (gtk_text_iter_has_tag (&iter, pix_tag))
         {
+          gtk_text_iter_backward_to_tag_toggle (&iter, pix_tag);
+          g_assert (gtk_text_iter_begins_tag (&iter, pix_tag));
           const int offset (gtk_text_iter_get_offset (&iter));
           GtkTextBuffer * buf (gtk_text_view_get_buffer (GTK_TEXT_VIEW(w)));
           const bool fullsize (toggle_fullsize_flag (buf));
-          resize_picture_at_iter (buf, &iter, fullsize, &w->allocation, tag);
+          resize_picture_at_iter (buf, &iter, fullsize, &w->allocation, pix_tag);
           gtk_text_iter_set_offset (&iter, offset);
           set_cursor_from_iter (event->window, w, &iter);
         }
@@ -593,7 +612,7 @@ namespace
   void
   append_text_buffer_nolock (const TextMassager  * text_massager,
                              GtkTextBuffer       * buffer,
-                             const char          * body_pch,
+                             const StringView    & body_in,
                              bool                  mute_quotes,
                              bool                  show_smilies,
                              bool                  do_markup)
@@ -601,11 +620,12 @@ namespace
     g_return_if_fail (buffer!=0);
     g_return_if_fail (GTK_IS_TEXT_BUFFER(buffer));
 
-
     // mute the quoted text, if desired
-    if (!body_pch) body_pch = "";
-    std::string body (mute_quotes ? text_massager->mute_quotes (body_pch) : body_pch);
-    //char * pan_find_signature_delimiter (body, &sig_point);
+    std::string body;
+    if (mute_quotes)
+      body = text_massager->mute_quotes (body_in);
+    else
+      body.assign (body_in.str, body_in.len);
 
     // insert the text
     GtkTextIter end;
@@ -774,9 +794,9 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
   {
     GdkPixbuf * original (get_pixbuf_from_gmime_part (part));
     const bool fullsize (!_prefs.get_flag ("size-pictures-to-fit", true));
-    GdkPixbuf * view_pic (size_to_fit (original, fullsize ? 0 : widget_size));
+    GdkPixbuf * scaled (size_to_fit (original, fullsize ? 0 : widget_size));
 
-    if (view_pic != 0)
+    if (scaled != 0)
     {
       GtkTextIter iter;
 
@@ -791,23 +811,21 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
 
       // add the picture
       const int begin_offset (gtk_text_iter_get_offset (&iter));
-      gtk_text_buffer_insert_pixbuf (_buffer, &iter, view_pic);
-      const int end_offset (gtk_text_iter_get_offset (&iter));
+      gtk_text_buffer_insert_pixbuf (_buffer, &iter, scaled);
       gtk_text_buffer_insert (_buffer, &iter, "\n", -1);
-      GtkTextIter iter_end = iter;
-      gtk_text_iter_set_offset (&iter, begin_offset);
-      gtk_text_iter_set_offset (&iter_end, end_offset);
+      GtkTextIter iter_begin (iter);
+      gtk_text_iter_set_offset (&iter_begin, begin_offset);
 
       // hook onto the tag a reference to the original picture
       // so that we can resize it later if user resizes the text pane.
-      set_pixbuf_at_offset (_buffer, begin_offset, original);
+      set_pixbuf_at_offset (_buffer, begin_offset, original, scaled);
       set_fullsize_flag (_buffer, fullsize);
 
       GtkTextTagTable * tags (gtk_text_buffer_get_tag_table (_buffer));
       GtkTextTag * tag (gtk_text_tag_table_lookup (tags, "pixbuf"));
-      gtk_text_buffer_apply_tag (_buffer, tag, &iter, &iter_end);
+      gtk_text_buffer_apply_tag (_buffer, tag, &iter_begin, &iter);
 
-      g_object_unref (view_pic);
+      g_object_unref (scaled);
       g_object_unref (original);
 
       is_done = true;
@@ -818,20 +836,15 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
   else if (g_mime_content_type_is_type (type, "text", "*"))
   {
     const char * fallback_charset (_charset.c_str());
-    char * str = mime_part_to_utf8 (part, fallback_charset);
+    std::string str = mime_part_to_utf8 (part, fallback_charset);
 
-    const bool do_wrap (_prefs.get_flag ("wrap-article-body", false));
-    if (str && *str && do_wrap) {
-      std::string wrapped = _tm.fill (str);
-      g_free (str);
-      str = g_strdup (wrapped.c_str());
-    }
+    if (!str.empty() && _prefs.get_flag ("wrap-article-body", false))
+      str = _tm.fill (str);
 
     const bool do_mute (_prefs.get_flag ("mute-quoted-text", false));
     const bool do_smilies (_prefs.get_flag ("show-smilies-as-graphics", true));
     const bool do_markup (_prefs.get_flag ("show-text-markup", true));
     append_text_buffer_nolock (&_tm, _buffer, str, do_mute, do_smilies, do_markup);
-    g_free (str);
     is_done = true;
   }
 
@@ -871,16 +884,15 @@ namespace
                               const char    * fallback_charset)
   {
     const char * val (message ? g_mime_message_get_header (message, key) : "");
-    char * utf8_val (header_to_utf8 (val, fallback_charset));
+    const std::string utf8_val (header_to_utf8 (val, fallback_charset));
     char * e (0);
     if (strcmp (key, "From"))
-      e = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", utf8_val);
+      e = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", utf8_val.c_str());
     else {
       const StringView v = GNKSA :: get_short_author_name (utf8_val);
       e = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", v.to_string().c_str());
     }
     s += e;
-    g_free (utf8_val);
     g_free (e);
   }
 
@@ -894,11 +906,10 @@ namespace
     e = g_markup_printf_escaped ("<span weight=\"bold\">%s:</span> ", key_i18n);
     s += e;
     g_free (e);
-    char * utf8_val = header_to_utf8 (val, fallback_charset);
-    e = g_markup_printf_escaped ("%s\n", utf8_val ? utf8_val : "");
+    const std::string utf8_val (header_to_utf8 (val, fallback_charset));
+    e = g_markup_printf_escaped ("%s\n", utf8_val.c_str());
     s += e;
-    const size_t retval (g_utf8_strlen(key,-1) + g_utf8_strlen(utf8_val,-1) + 2);
-    g_free (utf8_val);
+    const size_t retval (g_utf8_strlen(key,-1) + g_utf8_strlen(utf8_val.c_str(),-1) + 2);
     g_free (e);
     return retval;
   }
@@ -919,7 +930,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
 {
   const char * fallback_charset (_charset.c_str());
 
-  // set the headers...
+  // manditory headers...
   std::string s;
   size_t w(0), l(0);
   l = add_header_line (s, message, _("Subject"), "Subject", fallback_charset);
@@ -928,6 +939,32 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   w = std::max (w, l);
   l = add_header_line (s, message, _("Date"), "Date", fallback_charset);
   w = std::max (w, l);
+
+  // conditional headers...
+  if (message) {
+    const StringView newsgroups (g_mime_message_get_header (message, "Newsgroups"));
+    if (newsgroups.strchr(',')) {
+      l = add_header_line (s, message, _("Newsgroups"), "Newsgroups", fallback_charset);
+      w = std::max (w, l);
+    }
+    const StringView followup_to (g_mime_message_get_header (message, "Followup-To"));
+    if (!followup_to.empty() && (followup_to!=newsgroups)) {
+      l = add_header_line (s, message, _("Followup-To"), "Followup-To", fallback_charset);
+      w = std::max (w, l);
+    }
+    const StringView reply_to (g_mime_message_get_header (message, "Reply-To"));
+    if (!reply_to.empty()) {
+      const StringView from (g_mime_message_get_header (message, "From"));
+      StringView f_addr, f_name, rt_addr, rt_name;
+      GNKSA :: do_check_from (from, f_addr, f_name, false);
+      GNKSA :: do_check_from (reply_to, rt_addr, rt_name, false);
+      if (f_addr != rt_addr) {
+        l = add_header_line (s, message, _("Reply-To"), "Reply-To", fallback_charset);
+        w = std::max (w, l);
+      }
+    }
+  }
+  
   s.resize (s.size()-1); // remove trailing linefeed
   gtk_label_set_markup (GTK_LABEL(_headers), s.c_str());
 
@@ -974,10 +1011,9 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
     gtk_text_buffer_get_end_iter (_buffer, &end);
     StringView line, v(headers);
     while (v.pop_token (line, '\n')) {
-      char * pch = header_to_utf8 (line, fallback_charset);
-      gtk_text_buffer_insert (_buffer, &end, pch, -1);
+      const std::string h (header_to_utf8 (line, fallback_charset));
+      gtk_text_buffer_insert (_buffer, &end, h.c_str(), h.size());
       gtk_text_buffer_insert (_buffer, &end, "\n", 1);
-      g_free (pch);
     }
     gtk_text_buffer_insert (_buffer, &end, "\n", 1);
     g_free (headers);
@@ -1072,55 +1108,87 @@ BodyPane :: expander_activated_cb (GtkExpander*, gpointer self_gpointer)
   g_idle_add (expander_activated_idle, self_gpointer);
 }
 
+void
+BodyPane :: refresh_scroll_visible_state ()
+{
+  GtkScrolledWindow * w (GTK_SCROLLED_WINDOW (_scroll));
+  GtkAdjustment * adj = gtk_scrolled_window_get_hadjustment (w);
+  _hscroll_visible = adj->page_size < adj->upper;
+  adj = gtk_scrolled_window_get_vadjustment (w);
+  _vscroll_visible = adj->page_size < adj->upper;
+}
+
+// show*_cb exist to ensure that _hscroll_visible and _vscroll_visible
+// are initialized properly when the body pane becomes shown onscreen.
+
+gboolean
+BodyPane :: show_idle_cb (gpointer pane)
+{
+  static_cast<BodyPane*>(pane)->refresh_scroll_visible_state ();
+  return false;
+}
+void
+BodyPane :: show_cb (GtkWidget* w, gpointer pane)
+{
+  g_idle_add (show_idle_cb, pane);
+}
+
 namespace
 {
   guint text_size_allocated_idle_tag (0);
+}
 
-  gboolean text_size_allocated_idle (gpointer p)
-  {
-    GtkWidget * text (GTK_WIDGET (p));
+gboolean
+BodyPane :: text_size_allocated_idle_cb (gpointer pane)
+{
+  static_cast<BodyPane*>(pane)->text_size_allocated_idle ();
+  text_size_allocated_idle_tag = 0;
+  return false;
+}
 
-    // get the resize flag...
-    GtkTextBuffer * buf (gtk_text_view_get_buffer (GTK_TEXT_VIEW(text)));
-    const bool fullsize (get_fullsize_flag (buf));
+void
+BodyPane :: text_size_allocated_idle ()
+{
+  // prevent oscillation
+  const bool old_h (_hscroll_visible);
+  const bool old_v (_vscroll_visible);
+  refresh_scroll_visible_state ();
+  if ((old_h!=_hscroll_visible) || (old_v!=_vscroll_visible))
+    return;
 
-    // get the start point...
-    GtkTextIter iter;
-    gtk_text_buffer_get_start_iter (buf, &iter);
+  // get the resize flag...
+  GtkTextBuffer * buf (gtk_text_view_get_buffer (GTK_TEXT_VIEW(_text)));
+  const bool fullsize (get_fullsize_flag (buf));
 
-    // walk through the buffer looking for pictures to resize
-    GtkTextTag * tag (get_named_tag_from_view (text, "pixbuf"));
-    for (;;) {
-      if (gtk_text_iter_begins_tag (&iter, tag))
-        resize_picture_at_iter (buf, &iter, fullsize, &text->allocation, tag);
-      if (!gtk_text_iter_forward_to_tag_toggle (&iter, tag))
-        break;
-    }
+  // get the start point...
+  GtkTextIter iter;
+  gtk_text_buffer_get_start_iter (buf, &iter);
 
-    text_size_allocated_idle_tag = 0;
-    return false;
+  // walk through the buffer looking for pictures to resize
+  GtkTextTag * tag (get_named_tag_from_view (_text, "pixbuf"));
+  for (;;) {
+    if (gtk_text_iter_begins_tag (&iter, tag))
+      resize_picture_at_iter (buf, &iter, fullsize, &_text->allocation, tag);
+    if (!gtk_text_iter_forward_char (&iter))
+      break;
+    if (!gtk_text_iter_forward_to_tag_toggle (&iter, tag))
+      break;
   }
 }
 
 void
 BodyPane :: text_size_allocated (GtkWidget     * text,
                                  GtkAllocation * allocation,
-                                 gpointer        unused)
+                                 gpointer        pane)
 {
   if (!text_size_allocated_idle_tag)
-       text_size_allocated_idle_tag = g_idle_add (text_size_allocated_idle, text);
+       text_size_allocated_idle_tag = g_idle_add (text_size_allocated_idle_cb, pane);
 }
 
 /***
 ****
 ***/
 
-namespace {
-  void clear_pointer_on_destroy_signal (GtkWidget*& w) {
-    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
-  }
-}
-
 BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   _prefs (prefs),
   _data (data),
@@ -1148,6 +1216,7 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   gtk_object_sink (GTK_OBJECT(_terse));
   gtk_misc_set_alignment (GTK_MISC(_terse), 0.0f, 0.5f);
   gtk_label_set_use_markup (GTK_LABEL(_terse), true);
+  //gtk_label_set_selectable (GTK_LABEL(_terse), true);
 #if GTK_CHECK_VERSION(2,6,0)
   gtk_label_set_ellipsize (GTK_LABEL(_terse), PANGO_ELLIPSIZE_MIDDLE);
 #endif
@@ -1157,6 +1226,7 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   g_object_ref (_verbose);
   gtk_object_sink (GTK_OBJECT(_verbose));
   w = _headers = gtk_label_new ("Headers");
+  //gtk_label_set_selectable (GTK_LABEL(_headers), TRUE);
   gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
 #if GTK_CHECK_VERSION(2,6,0)
   gtk_label_set_ellipsize (GTK_LABEL(w), PANGO_ELLIPSIZE_MIDDLE);
@@ -1175,25 +1245,19 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   gtk_container_set_border_width (GTK_CONTAINER(_text), PAD_SMALL);
   gtk_text_view_set_editable (GTK_TEXT_VIEW(_text), false);
   gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW(_text), false);
-  GtkWidget * scroll = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
+  _scroll = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(_scroll), GTK_SHADOW_IN);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (_scroll),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER(scroll), _text);
+  gtk_container_add (GTK_CONTAINER(_scroll), _text);
   gtk_widget_show_all (vbox);
-  gtk_box_pack_start (GTK_BOX(vbox), scroll, true, true, 0);
+  gtk_box_pack_start (GTK_BOX(vbox), _scroll, true, true, 0);
 
   // set up the buffer tags
   _buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(_text));
   set_text_buffer_tags (_buffer, _prefs);
 
-  // listen for user interaction
-  g_signal_connect (_text, "motion_notify_event", G_CALLBACK(motion_notify_event), 0);
-  g_signal_connect (_text, "button_press_event", G_CALLBACK(mouse_button_pressed), 0);
-  g_signal_connect (_text, "key_press_event", G_CALLBACK(text_key_pressed), scroll);
-  g_signal_connect (_text, "size_allocate", G_CALLBACK(text_size_allocated), this);
-
   set_text_from_message (0);
   const bool expanded (_prefs.get_flag ("body-pane-headers-expanded", true));
   gtk_expander_set_expanded (GTK_EXPANDER(_expander), expanded);
@@ -1202,13 +1266,12 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   _root = vbox;
   _prefs.add_listener (this);
 
-  clear_pointer_on_destroy_signal (_expander);
-  clear_pointer_on_destroy_signal (_terse);
-  clear_pointer_on_destroy_signal (_verbose);
-  clear_pointer_on_destroy_signal (_headers);
-  clear_pointer_on_destroy_signal (_xface);
-  clear_pointer_on_destroy_signal (_root);
-  clear_pointer_on_destroy_signal (_text);
+  // listen for user interaction
+  g_signal_connect (_text, "motion_notify_event", G_CALLBACK(motion_notify_event), 0);
+  g_signal_connect (_text, "button_press_event", G_CALLBACK(mouse_button_pressed), &_prefs);
+  g_signal_connect (_text, "key_press_event", G_CALLBACK(text_key_pressed), _scroll);
+  g_signal_connect (_text, "size_allocate", G_CALLBACK(text_size_allocated), this);
+  g_signal_connect (_root, "show", G_CALLBACK(show_cb), this);
 
   gtk_widget_show_all (_root);
 }
@@ -1270,8 +1333,40 @@ BodyPane :: read_more_or_less (bool more)
   return handled;
 }
 
+namespace
+{
+  // "Re: RE: Re: Re: foo" --> "Re: foo"
+  StringView normalize_subject_re (const StringView& v_in)
+  {
+    StringView ret, v(v_in);
+    for (;;) {
+      v.ltrim ();
+      ret = v;
+      if (v.strstr("Re:") == v.str)
+        v.eat_chars (3);
+      else if (v.strstr("RE:") == v.str)
+        v.eat_chars (3);
+      else
+        break;
+    }
+    return ret;
+  }
+
+  std::string get_header (GMimeMessage * msg,
+                          const char   * key, 
+                          const char   * fallback_charset_1,
+                          const char   * fallback_charset_2)
+  {
+    const StringView v (g_mime_message_get_header (msg, key));
+    std::string s;
+    if (!v.empty())
+      s = header_to_utf8 (v, fallback_charset_1, fallback_charset_2);
+    return s;
+  }
+}
+
 GMimeMessage*
-BodyPane :: create_followup ()
+BodyPane :: create_followup_or_reply (bool is_reply)
 {
   GMimeMessage * msg (0);
 
@@ -1279,33 +1374,56 @@ BodyPane :: create_followup ()
   {
     msg = g_mime_message_new (false);
 
+    // fallback character encodings
+    const char * group_charset (_charset.c_str());
+    const GMimeContentType * type (g_mime_object_get_content_type (GMIME_OBJECT(_message)));
+    const char * message_charset (type ? g_mime_content_type_get_parameter (type, "charset") : 0);
+
     ///
     ///  HEADERS
     ///
 
-    // newsgroups
-    const char * header = "Newsgroups";
-    StringView v (g_mime_message_get_header (_message, header));
-    g_mime_message_add_header (msg, header, v.str);
+    // To:, Newsgroups:
+    const std::string from       (get_header (_message, "From",        message_charset, group_charset));
+    const std::string newsgroups (get_header (_message, "Newsgroups",  message_charset, group_charset));
+    const std::string fup_to     (get_header (_message, "Followup-To", message_charset, group_charset));
+    const std::string reply_to   (get_header (_message, "Reply-To",    message_charset, group_charset));
+    if (is_reply || fup_to=="poster") {
+      const std::string& to (reply_to.empty() ? from : reply_to);
+      g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.c_str());
+    } else {
+      const std::string& groups (fup_to.empty() ? newsgroups : fup_to);
+      g_mime_message_add_header (msg, "Newsgroups", groups.c_str());
+    }
 
-    // subject
-    v = g_mime_message_get_subject (_message);
+    // Subject:
+    StringView v = g_mime_message_get_subject (_message);
+    v = normalize_subject_re (v);
     std::string val (v.str, v.len);
-    if (val.find ("Re:") != 0)
-      val.insert (0, "Re: ");
-    g_mime_message_set_subject (msg, val.c_str());
+    if (!val.find("RE:") || !val.find ("Re:"))
+      val.replace (0, 3, "Re:"); // be polite & force lowercase 'e'
+    else
+      val.insert (0, "Re: "); // no Re: -- add one.
+    std::string h = header_to_utf8 (val, message_charset, group_charset);
+    g_mime_message_set_subject (msg, h.c_str());
+
+    // attribution lines
 
-    // attribution line
     const char * cpch = g_mime_message_get_sender (_message);
-    g_mime_message_add_header (msg, "X-Draft-Attribution-Author", cpch);
+    h = header_to_utf8 (cpch, message_charset, group_charset);
+    g_mime_message_add_header (msg, "X-Draft-Attribution-Author", h.c_str());
+
     cpch = g_mime_message_get_message_id (_message);
-    g_mime_message_add_header (msg, "X-Draft-Attribution-Id", cpch);
-    char * pch = g_mime_message_get_date_string (_message);
-    g_mime_message_add_header (msg, "X-Draft-Attribution-Date", pch);
-    g_free (pch);
+    h = header_to_utf8 (cpch, message_charset, group_charset);
+    g_mime_message_add_header (msg, "X-Draft-Attribution-Id", h.c_str());
+
+    char * tmp = g_mime_message_get_date_string (_message);
+    h = header_to_utf8 (tmp, message_charset, group_charset);
+    g_mime_message_add_header (msg, "X-Draft-Attribution-Date", h.c_str());
+    g_free (tmp);
 
     // references
-    header = "References";
+    const char * header = "References";
     v = g_mime_message_get_header (_message, header);
     val.assign (v.str, v.len);
     if (!val.empty())
@@ -1324,30 +1442,31 @@ BodyPane :: create_followup ()
     if (gtk_text_buffer_get_selection_bounds (_buffer, &start, &end))
     {
       // go with the user-selected region w/o modifications
-      pch = gtk_text_buffer_get_text (_buffer, &start, &end, false);
+      h = gtk_text_buffer_get_text (_buffer, &start, &end, false);
     }
     else
     {
       // get the entire body...
       gboolean unused (false);
-      pch = g_mime_message_get_body (_message, true, &unused);
+      char * pch = g_mime_message_get_body (_message, true, &unused);
+      h = content_to_utf8 (pch, message_charset, group_charset);
+      g_free (pch);
 
       // remove the sig...
       int sig_index (0);
-      if (GNKSA::find_signature_delimiter (pch, sig_index) != GNKSA::SIG_NONE)
-        pch[sig_index] = '\0';
+      if (GNKSA::find_signature_delimiter (h, sig_index) != GNKSA::SIG_NONE)
+        h[sig_index] = '\0';
 
       // trim...
-      StringView v (pch);
+      StringView v (h);
       v.trim ();
-      std::memmove (pch, v.str, v.len);
-      pch[v.len] = '\0';
+      h = std::string(v.str,v.len);
     }
 
     // quote the body
     std::string s;
-    for (const char *c(pch); c && *c; ++c) {
-      if (c==pch || c[-1]=='\n')
+    for (const char *c(h.c_str()); c && *c; ++c) {
+      if (c==h.c_str() || c[-1]=='\n')
         s += (*c=='>' ? ">" : "> ");
       s += *c;
     }
@@ -1357,15 +1476,14 @@ BodyPane :: create_followup ()
     g_mime_stream_write_string (stream, s.c_str());
     GMimeDataWrapper * wrapper = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_8BIT);
     GMimePart * part = g_mime_part_new ();
-    GMimeContentType * type = g_mime_content_type_new_from_string ("text/plain; charset=UTF-8");
-    g_mime_part_set_content_type (part, type);
+    GMimeContentType * new_type = g_mime_content_type_new_from_string ("text/plain; charset=UTF-8");
+    g_mime_part_set_content_type (part, new_type);
     g_mime_part_set_content_object (part, wrapper);
     g_mime_part_set_encoding (part, GMIME_PART_ENCODING_8BIT);
     g_mime_message_set_mime_part (msg, GMIME_OBJECT(part));
     g_object_unref (wrapper);
     g_object_unref (part);
     g_object_unref (stream);
-    g_free (pch);
 //std::cerr << LINE_ID << " here is the modified clone\n" << g_mime_message_to_string(msg) << std::endl;
   }
 
