@@ -846,12 +846,6 @@ GroupPane :: on_selection_changed (GtkTreeSelection * sel, gpointer pane_gpointe
   self->_action_manager.sensitize_action ("show-group-preferences-dialog", is_group(group));
 }
 
-namespace {
-  void clear_pointer_on_destroy_signal (GtkWidget*& w) {
-    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
-  }
-}
-
 GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs):
   _prefs (prefs),
   _data (data),
@@ -914,9 +908,6 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   _prefs.add_listener (this);
 
   refresh_font ();
-
-  clear_pointer_on_destroy_signal (_root);
-  clear_pointer_on_destroy_signal (_tree_view);
 }
 
 GroupPane :: ~GroupPane ()
