@@ -38,6 +38,7 @@ extern "C" {
 #include "group-pane.h"
 #include "group-prefs-dialog.h"
 #include "header-pane.h"
+#include "hig.h"
 #include "license.h"
 #include "log-ui.h"
 #include "gui.h"
@@ -497,7 +498,7 @@ namespace
   }
 }
 
-void GUI :: do_save_attachments ()
+void GUI :: do_save_articles ()
 {
   std::string path;
   std::set<const Article*> selected_articles (_header_pane->get_full_selection ());
@@ -882,14 +883,132 @@ GUI :: score_add (int mode)
   gtk_widget_show (d->root());
 }
 
-void GUI :: do_cancel_article ()
-{
-  std::cerr << "FIXME " << LINE_ID << std::endl;
-}
 void GUI :: do_supersede_article ()
 {
-  std::cerr << "FIXME " << LINE_ID << std::endl;
+  GMimeMessage * message (_body_pane->get_message ());
+  if (!message)
+    return;
+
+  // did this user post the message?
+  const char * sender (g_mime_message_get_sender (message));
+  const bool user_posted_this (_data.has_from_header (sender));
+  if (!user_posted_this) {
+    GtkWidget * w = gtk_message_dialog_new (
+      get_window(_root),
+      GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+      GTK_MESSAGE_ERROR,
+      GTK_BUTTONS_CLOSE, NULL);
+    HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
+      _("Unable to supersede article."),
+      _("The article doesn't match any of your posting profiles."));
+    g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+    gtk_widget_show (w);
+    g_object_unref (message);
+    return;
+  }
+
+  // copy the body and preserve the essential headers...
+  // if we copy the entire message, then we get all the
+  // headers tacked on by the news server.
+  const char * cpch;
+  char * old_mid (g_strdup_printf ("<%s>", g_mime_message_get_message_id(message)));
+  GMimeMessage * new_message (g_mime_message_new (false));
+  g_mime_message_set_header (new_message, "Supersedes", old_mid);
+  g_mime_message_set_sender (new_message, g_mime_message_get_sender (message));
+  g_mime_message_set_subject (new_message, g_mime_message_get_subject (message));
+  g_mime_message_set_header (new_message, "Newsgroups", g_mime_message_get_header (message, "Newsgroups"));
+  if ((cpch = g_mime_message_get_reply_to (message)))
+              g_mime_message_set_reply_to (new_message, cpch);
+  if ((cpch = g_mime_message_get_header (message,     "Followup-To")))
+              g_mime_message_set_header (new_message, "Followup-To", cpch);
+  gboolean  unused (false);
+  char * body (g_mime_message_get_body (message, true, &unused));
+  GMimeStream * stream = g_mime_stream_mem_new_with_buffer (body, strlen(body));
+  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_DEFAULT);
+  GMimePart * part = g_mime_part_new ();
+  g_mime_part_set_content_object (part, content_object);
+  g_mime_message_set_mime_part (new_message, GMIME_OBJECT(part));
+  g_object_unref (part);
+  g_object_unref (content_object);
+  g_object_unref (stream);
+
+  PostUI * post = new PostUI (_data, _queue, _data, _data, new_message, _prefs, _group_prefs);
+  gtk_widget_show (post->root());
+
+  GtkWidget * w = gtk_message_dialog_new (
+    GTK_WINDOW(post->root()),
+    GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+    GTK_MESSAGE_INFO,
+    GTK_BUTTONS_CLOSE, NULL);
+  HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
+    _("Revise and send this article to replace the old one."),
+    _("Be patient!  It will take time for your changes to take effect."));
+  g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+  gtk_widget_show (w);
+
+  g_object_unref (message);
+  g_free (old_mid);
 }
+
+void GUI :: do_cancel_article ()
+{
+  GMimeMessage * message (_body_pane->get_message ());
+  if (!message)
+    return;
+
+  // did this user post the message?
+  const char * sender (g_mime_message_get_sender (message));
+  const bool user_posted_this (_data.has_from_header (sender));
+  if (!user_posted_this) {
+    GtkWidget * w = gtk_message_dialog_new (
+      get_window(_root),
+      GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+      GTK_MESSAGE_ERROR,
+      GTK_BUTTONS_CLOSE, NULL);
+    HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
+      _("Unable to cancel article."),
+      _("The article doesn't match any of your posting profiles."));
+    g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+    gtk_widget_show (w);
+    g_object_unref (message);
+    return;
+  }
+
+  // okay then...
+  GMimeMessage * cancel = g_mime_message_new (false);
+  char * cancel_message = g_strdup_printf ("cancel <%s>", g_mime_message_get_message_id(message));
+  g_mime_message_set_sender (cancel, g_mime_message_get_sender (message));
+  g_mime_message_set_subject (cancel, "Cancel");
+  g_mime_message_set_header (cancel, "Newsgroups", g_mime_message_get_header (message, "Newsgroups"));
+  g_mime_message_set_header (cancel, "Control", cancel_message);
+  const char * body ("Ignore\r\nArticle canceled by author using " PACKAGE_STRING "\r\n");
+  GMimeStream * stream = g_mime_stream_mem_new_with_buffer (body, strlen(body));
+  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_DEFAULT);
+  GMimePart * part = g_mime_part_new ();
+  g_mime_part_set_content_object (part, content_object);
+  g_mime_message_set_mime_part (cancel, GMIME_OBJECT(part));
+  g_object_unref (part);
+  g_object_unref (content_object);
+  g_object_unref (stream);
+  g_free (cancel_message);
+
+  PostUI * post = new PostUI (_data, _queue, _data, _data, cancel, _prefs, _group_prefs);
+  gtk_widget_show (post->root());
+
+  GtkWidget * w = gtk_message_dialog_new (
+    GTK_WINDOW(post->root()),
+    GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+    GTK_MESSAGE_INFO,
+    GTK_BUTTONS_CLOSE, NULL);
+  HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(w),
+    _("Send this article to ask your server to cancel your other one."),
+    _("Be patient!  It will take time for your changes to take effect."));
+  g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+  gtk_widget_show (w);
+
+  g_object_unref (message);
+}
+
 void GUI :: do_delete_article ()
 {
   const std::set<const Article*> articles (_header_pane->get_nested_selection());
@@ -928,31 +1047,41 @@ GUI :: do_post ()
   g_mime_message_set_mime_part (message, GMIME_OBJECT(part));
   g_object_unref (part);
 
-  PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs, get_window(_root));
+  PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs);
   gtk_widget_show (post->root());
   g_object_unref (message);
 }
 
-void GUI :: do_followup ()
+void GUI :: do_followup_to ()
 {
-  GMimeMessage * message = _body_pane->create_followup ();
+  GMimeMessage * message = _body_pane->create_followup_or_reply (false);
   if (message) {
-    PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs, get_window(_root));
+    PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs);
     gtk_widget_show (post->root());
     g_object_unref (message);
   }
 }
+void GUI :: do_reply_to ()
+{
+  GMimeMessage * message = _body_pane->create_followup_or_reply (true);
+  if (message) {
+    PostUI * post = new PostUI (_data, _queue, _data, _data, message, _prefs, _group_prefs);
+    gtk_widget_show (post->root());
+    g_object_unref (message);
+  }
+}
+
 void GUI :: do_pan_web ()
 {
-  pan_url_show ("http://pan.rebelbase.com/");
+  URL :: open (_prefs, "http://pan.rebelbase.com/");
 }
 void GUI :: do_bug_report ()
 {
-  pan_url_show ("http://bugzilla.gnome.org/enter_bug.cgi?product=Pan");
+  URL :: open (_prefs, "http://bugzilla.gnome.org/enter_bug.cgi?product=Pan");
 }
 void GUI :: do_tip_jar ()
 {
-  pan_url_show ("http://pan.rebelbase.com/tipjar/");
+  URL :: open (_prefs, "http://pan.rebelbase.com/tipjar/");
 }
 void GUI :: do_about_pan ()
 {
