@@ -68,7 +68,7 @@ namespace pan
       virtual void on_prefs_color_changed (const StringView& key, const GdkColor& value) { }
 
     public: // PanUI
-      virtual void do_save_attachments ();
+      virtual void do_save_articles ();
       virtual void do_print ();
       virtual void do_quit ();
       virtual void do_import_tasks ();
@@ -116,7 +116,8 @@ namespace pan
       virtual void do_mark_article_read ();
       virtual void do_mark_article_unread ();
       virtual void do_post ();
-      virtual void do_followup ();
+      virtual void do_followup_to ();
+      virtual void do_reply_to ();
       virtual void do_pan_web ();
       virtual void do_bug_report ();
       virtual void do_tip_jar ();
