@@ -781,8 +781,9 @@ struct HeaderPane::SelectFirstArticle: public PanTreeStore::WalkFunctor
   GtkTreeView * tree_view;
   GtkTreeSelection * tree_selection;
   const articles_t& articles;
-  SelectFirstArticle (GtkTreeView * v, GtkTreeSelection *sel, const articles_t& a):
-    tree_view(v), tree_selection(sel), articles(a) {}
+  const bool do_scroll;
+  SelectFirstArticle (GtkTreeView * v, GtkTreeSelection *sel, const articles_t& a, bool scroll):
+    tree_view(v), tree_selection(sel), articles(a), do_scroll(scroll) {}
   virtual ~SelectFirstArticle () {}
   virtual bool operator()(PanTreeStore *store, PanTreeStore::Row* r, GtkTreeIter *iter, GtkTreePath *unused) {
     Row * row (dynamic_cast<Row*>(r));
@@ -791,9 +792,11 @@ struct HeaderPane::SelectFirstArticle: public PanTreeStore::WalkFunctor
       GtkTreePath * path = gtk_tree_model_get_path (GTK_TREE_MODEL(store), iter);
       gtk_tree_view_expand_row (tree_view, path, true);
       gtk_tree_view_expand_to_path (tree_view, path);
-      //gtk_tree_view_set_cursor (tree_view, path, NULL, false);
-      //gtk_tree_view_scroll_to_cell (tree_view, path, NULL, true, 0.5f, 0.0f);
-      //gtk_tree_selection_select_path (tree_selection, path);
+      if (do_scroll) {
+        gtk_tree_view_set_cursor (tree_view, path, NULL, false);
+        gtk_tree_view_scroll_to_cell (tree_view, path, NULL, true, 0.5f, 0.0f);
+        gtk_tree_selection_select_path (tree_selection, path);
+      }
       gtk_tree_path_free (path);
       return false;
     }
@@ -808,7 +811,11 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
       << diffs.reparented.size() << " reparented; "
       << diffs.removed.size() << " removed");
 
-  _tree_store->pause_sorting ();
+  const bool do_sort (!diffs.added.empty()
+                      || !diffs.reparented.empty()
+                      || !diffs.removed.empty());
+  if (do_sort)
+    _tree_store->pause_sorting ();
 
   // figure out which articles to select when this is all done.
   // if none of the current selections survive,
@@ -870,12 +877,18 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   if (!diffs.added.empty() && _prefs.get_flag ("expand-threads-when-entering-group", false))
     gtk_tree_view_expand_all (GTK_TREE_VIEW(_tree_view));
 
-  _tree_store->resume_sorting ();
+  if (do_sort)
+    _tree_store->resume_sorting ();
 
   // select the next article if one was deleted...
+  // when an xover task adds a batch of articles periodically,
+  // scrolling causes an irritating 'jump', but when you delete
+  // articles, you /want/ that jump to the next article.
+  // So, scroll in the latter case but not the former.
   if (!selectme.empty()) {
+    const bool do_scroll (diffs.added.empty() && !diffs.removed.empty());
     GtkTreeSelection * sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view));
-    SelectFirstArticle walk (GTK_TREE_VIEW(_tree_view), sel, selectme);
+    SelectFirstArticle walk (GTK_TREE_VIEW(_tree_view), sel, selectme, do_scroll);
     _tree_store->walk (walk);
   }
 }
@@ -1125,7 +1138,7 @@ namespace
       else if (is_mediumish && is_pictures_newsgroup)
         pane->_action_manager.activate_action ("read-selected-article");
       else
-        pane->_action_manager.activate_action ("save-attachments");
+        pane->_action_manager.activate_action ("save-articles");
     }
     return false;
   }
@@ -1420,6 +1433,13 @@ namespace
                     0, 0,
                     0, gtk_get_current_event_time());
   }
+
+  void ellipsize_if_supported (GObject * o)
+  {
+#if GTK_CHECK_VERSION(2,6,0)
+    g_object_set (o, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+#endif
+  }
 }
 
 void
@@ -1477,10 +1497,8 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "ypad", 0,
         "xpad", xpad,
-#       if GTK_CHECK_VERSION(2,6,0)
-        "ellipsize", PANGO_ELLIPSIZE_END,
-#       endif
         NULL));
+      ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 400));
@@ -1496,10 +1514,8 @@ HeaderPane :: build_tree_columns ()
         "ypad", 0,
         "xpad", xpad,
         "xalign", 1.0,
-#       if GTK_CHECK_VERSION(2,6,0)
-        "ellipsize", PANGO_ELLIPSIZE_END,
-#       endif
         NULL));
+      ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Score"), r, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 50));
@@ -1513,10 +1529,8 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-#       if GTK_CHECK_VERSION(2,6,0)
-        "ellipsize", PANGO_ELLIPSIZE_END,
-#       endif
         NULL));
+      ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Author"), r, "text", COL_SHORT_AUTHOR, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 133));
@@ -1530,10 +1544,8 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-#       if GTK_CHECK_VERSION(2,6,0)
-        "ellipsize", PANGO_ELLIPSIZE_END,
-#       endif
         NULL));
+      ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Lines"), r, "text", COL_LINES, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 60));
@@ -1547,10 +1559,8 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-#       if GTK_CHECK_VERSION(2,6,0)
-        "ellipsize", PANGO_ELLIPSIZE_END,
-#       endif
         NULL));
+      ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Bytes"), r, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 80));
@@ -1564,10 +1574,8 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-#       if GTK_CHECK_VERSION(2,6,0)
-        "ellipsize", PANGO_ELLIPSIZE_END,
-#       endif
         NULL));
+      ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Date"), r, "text", COL_DATE_STR, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 120));
@@ -1674,12 +1682,6 @@ HeaderPane :: create_filter_entry ()
 
 Data::ShowType _show_type;
 
-namespace {
-  void clear_pointer_on_destroy_signal (GtkWidget*& w) {
-    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
-  }
-}
-
 HeaderPane :: HeaderPane (ActionManager       & action_manager,
                           Data                & data,
                           Queue               & queue,
@@ -1733,9 +1735,6 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   _cache.add_listener (this);
 
   refresh_font ();
-
-  clear_pointer_on_destroy_signal (_root);
-  clear_pointer_on_destroy_signal (_tree_view);
 }
 
 void
