@@ -362,10 +362,12 @@ PanTreeStore :: sortable_has_sort_func (GtkTreeSortable *sortable, gint col)
   return (it!=tree->sort_info->end()) && (it->second.sort_func!=0);
 }
 
+// SORTING
+
 struct
-PanTreeStore :: SortTuple
+PanTreeStore :: SortRowInfo
 {
-  int offset;
+  int oldpos;
   Row * row;
   GtkTreeIter iter;
 };
@@ -383,12 +385,12 @@ struct PanTreeStore :: SortHelperData
 };
 
 int
-PanTreeStore :: tuple_compare_func (gconstpointer a_gpointer,
-                                    gconstpointer b_gpointer,
-                                    gpointer user_data)
+PanTreeStore :: row_compare_func (gconstpointer a_gpointer,
+                                  gconstpointer b_gpointer,
+                                  gpointer      user_data)
 {
-  const SortTuple * a = (const SortTuple*) a_gpointer;
-  const SortTuple * b = (const SortTuple*) b_gpointer;
+  const SortRowInfo * a = (const SortRowInfo*) a_gpointer;
+  const SortRowInfo * b = (const SortRowInfo*) b_gpointer;
   const SortHelperData * help = (const SortHelperData*) user_data;
   int val = (help->sort_info.sort_func) (help->model,
                                          const_cast<GtkTreeIter*>(&a->iter),
@@ -398,46 +400,15 @@ PanTreeStore :: tuple_compare_func (gconstpointer a_gpointer,
     val = -val;
 
   if (!val) // inplace sort
-    val = help->store->get_row(&a->iter)->child_index - 
-          help->store->get_row(&b->iter)->child_index;
+    val = a->row->child_index - b->row->child_index;
 
   return val;
 }
 
 void
-PanTreeStore :: renumber_children (Row * parent,
-                                   int child_lo,
-                                   int child_hi)
-{
-  const int n_children (parent->n_children());
-  child_hi = std::min (child_hi, n_children);
-
-  g_assert (parent);
-  g_assert (child_lo >= 0);
-  g_assert (child_lo <= child_hi);
-  g_assert (child_hi <= n_children);
-
-  Row ** it = &parent->children[child_lo];
-  for (int i=child_lo; i!=child_hi; ++i, ++it) {
-    (*it)->child_index = i;
-    (*it)->parent = parent;
-  }
-
-#if 0
-  // just for testing.. these lines slow things down
-  // and will be commented out
-  for (int i=0; i<n_children; ++i) {
-    g_assert (parent->children[i]->child_index == i);
-    g_assert (parent->children[i]->parent == parent);
-  }
-#endif
-}
-
-
-void
-PanTreeStore :: sort_helper (SortInfo  & sort_info,
-                             Row       * parent,
-                             bool        recurse)
+PanTreeStore :: sort_children (SortInfo  & sort_info,
+                               Row       * parent,
+                               bool        recurse)
 {
   g_assert (parent);
 
@@ -446,20 +417,18 @@ PanTreeStore :: sort_helper (SortInfo  & sort_info,
     return;
 
   // build a temporary array to sort
-  SortTuple * sort_array = new SortTuple [n];
+  SortRowInfo * sort_array = new SortRowInfo [n];
   for (int i=0; i<n; ++i) {
-    SortTuple& tuple = sort_array[i];
-    tuple.offset = i;
-    tuple.row = parent->children[i];
-    set_iter (&tuple.iter, tuple.row);
+    SortRowInfo& row_info = sort_array[i];
+    row_info.oldpos = i;
+    row_info.row = parent->children[i];
+    set_iter (&row_info.iter, row_info.row);
   }
 
   // sort 'em...
   SortHelperData help (this, sort_info, order);
-  g_qsort_with_data (sort_array, n,
-                     sizeof(SortTuple),
-                     tuple_compare_func,
-                     &help);
+  g_qsort_with_data (sort_array, n, sizeof(SortRowInfo),
+                     row_compare_func, &help);
 
   // update the child indices...
   for (int i=0; i<n; ++i) {
@@ -471,33 +440,34 @@ PanTreeStore :: sort_helper (SortInfo  & sort_info,
 
   // let the world know we've changed
   int * new_order (new int [n]);
-  bool differ (false);
+  bool reordered (false);
   for (int i=0; i<n; ++i) {
-    differ |= (i != sort_array[i].offset);
-    new_order[i] = sort_array[i].offset;
+    reordered |= (i != sort_array[i].oldpos);
+    new_order[i] = sort_array[i].oldpos;
   }
 
   // tell the world...
-  GtkTreeIter tmp_iter, *parent_iter;
-  GtkTreePath * path;
-  if (parent == root) {
-    parent_iter = 0;
-    path = gtk_tree_path_new ();
-  } else {
-    set_iter (&tmp_iter, parent);
-    parent_iter = &tmp_iter;
-    path = gtk_tree_model_get_path (help.model, parent_iter);
+  if (reordered) {
+    GtkTreeIter *parent_iter;
+    GtkTreePath * path;
+    if (parent == root) {
+      parent_iter = 0;
+      path = gtk_tree_path_new ();
+    } else {
+      GtkTreeIter tmp_iter;
+      set_iter (&tmp_iter, parent);
+      parent_iter = &tmp_iter;
+      path = gtk_tree_model_get_path (help.model, parent_iter);
+    }
+    gtk_tree_model_rows_reordered (help.model, path, parent_iter, new_order);
+    gtk_tree_path_free (path);
   }
-  gtk_tree_model_rows_reordered (help.model, path, parent_iter, new_order);
-  gtk_tree_path_free (path);
-  delete [] new_order;
 
+  delete [] new_order;
   delete [] sort_array;
 
-  for (int i=0; recurse && i<n; ++i) {
-    Row * child = parent->children[i];
-    sort_helper (sort_info, child, recurse);
-  }
+  for (int i=0; recurse && i<n; ++i)
+    sort_children (sort_info, parent->children[i], recurse);
 }
 
 void
@@ -507,7 +477,7 @@ PanTreeStore :: sort ()
   {
     g_assert (sortable_has_sort_func (GTK_TREE_SORTABLE(this), sort_column_id));
     SortInfo& info = (*sort_info)[sort_column_id];
-    sort_helper (info, root, true);
+    sort_children (info, root, true);
   }
 }
 
@@ -738,6 +708,26 @@ PanTreeStore :: row_changed (GtkTreeIter * iter)
 ****
 ***/
 
+void
+PanTreeStore :: renumber_children (Row * parent,
+                                   int child_lo,
+                                   int child_hi)
+{
+  const int n_children (parent->n_children());
+  child_hi = std::min (child_hi, n_children);
+
+  g_assert (parent);
+  g_assert (child_lo >= 0);
+  g_assert (child_lo <= child_hi);
+  g_assert (child_hi <= n_children);
+
+  Row ** it = &parent->children[child_lo];
+  for (int i=child_lo; i!=child_hi; ++i, ++it) {
+    (*it)->child_index = i;
+    (*it)->parent = parent;
+  }
+}
+
 typedef std::vector<PanTreeStore::Row*> rows_t;
 
 void
@@ -798,7 +788,7 @@ PanTreeStore :: insert_n (GtkTreeIter                     * iter,
     gtk_tree_path_next (path);
   }
 
-  // maybe emit the 'row has childed' signal
+  // maybe emit the 'row has child toggled' signal
   if (!old_size && parent_row!=root) {
     gtk_tree_path_up (path);
     gtk_tree_model_row_has_child_toggled (model, path, parent_it);
@@ -856,12 +846,11 @@ struct PanTreeStore::ReparentWalker: public PanTreeStore::WalkFunctor
         GtkTreeIter citer;
         store->set_iter (&citer, child);
         gtk_tree_model_row_inserted (model, cpath, &citer);
+        if (!i)
+          gtk_tree_model_row_has_child_toggled (model, path, iter);
         gtk_tree_path_next (cpath);
       }
       gtk_tree_path_free (cpath);
-
-      // fire a "has children toggled"
-      gtk_tree_model_row_has_child_toggled (model, path, iter);
     }
 
     return true; // keep marching
@@ -869,9 +858,9 @@ struct PanTreeStore::ReparentWalker: public PanTreeStore::WalkFunctor
 };
 
 void
-PanTreeStore :: reparent (Row            * new_parent,
-                          Row            * row,
-                          int              position)
+PanTreeStore :: reparent (Row  * new_parent,
+                          Row  * row,
+                          int    position)
 {
   g_return_if_fail (row != 0);
 
@@ -1031,7 +1020,7 @@ PanTreeStore :: clear ()
 void
 PanTreeStore :: remove_siblings (Row     ** siblings,
                                  size_t     n_siblings,
-                                 bool      free_memory)
+                                 bool       free_memory)
 {
   // entry assertions
   g_assert (siblings);
@@ -1042,16 +1031,9 @@ PanTreeStore :: remove_siblings (Row     ** siblings,
     g_assert (nit==siblings || ((*nit)->child_index < nit[-1]->child_index)); // sorted from high to low child index
   }
 
-  // get a path to the first sibilng.
-  // we'll reuse it for all siblings.
-  GtkTreeIter iter;
-  set_iter (&iter, siblings[0]);
-  GtkTreeModel * model (GTK_TREE_MODEL(this));
-  GtkTreePath * path = gtk_tree_model_get_path (model, &iter);
-  int * last_path_index = gtk_tree_path_get_indices (path);
-  last_path_index += gtk_tree_path_get_depth(path)-1;
-
   // walk through all the siblings and unthread them.
+  GtkTreeModel * model (GTK_TREE_MODEL(this));
+  GtkTreePath * path = get_path (parent);
   for (Row** nit(siblings), **nend(siblings+n_siblings); nit!=nend; ++nit)
   {
     Row * row (*nit);
@@ -1063,8 +1045,9 @@ PanTreeStore :: remove_siblings (Row     ** siblings,
     parent->children.erase (parent->children.begin() + pos);
 
     // let the world know the row was removed
-    *last_path_index = row->child_index;
+    gtk_tree_path_append_index (path, row->child_index);
     gtk_tree_model_row_deleted (model, path);
+    gtk_tree_path_up (path);
 
     // free `row' and all its descendants
     if (free_memory) {
@@ -1082,7 +1065,6 @@ PanTreeStore :: remove_siblings (Row     ** siblings,
   if (parent!=root && !parent->n_children()) {
     GtkTreeIter pit;
     set_iter (&pit, parent);
-    gtk_tree_path_up (path);
     gtk_tree_model_row_has_child_toggled (model, path, &pit);
   }
 
