@@ -402,10 +402,10 @@ struct PanTreeStore
     static gboolean sortable_has_default_sort_func (GtkTreeSortable*);
  
   private:
-    struct SortTuple;
+    struct SortRowInfo;
     struct SortHelperData;
-    void sort_helper (SortInfo&, Row* parent, bool recurse);
-    static int tuple_compare_func (gconstpointer, gconstpointer, gpointer);
+    void sort_children (SortInfo&, Row* parent, bool recurse);
+    static int row_compare_func (gconstpointer, gconstpointer, gpointer);
 
   private:
     PanTreeStore(); // this is a GObject; use new_tree instead
