@@ -18,6 +18,7 @@
  */
 
 #include <config.h>
+#include <signal.h>
 extern "C" {
   #include <glib/gi18n.h>
   #include <gtk/gtk.h>
@@ -48,6 +49,24 @@ namespace
     return true; // don't invoke the default handler that destroys the widget
   }
 
+#ifndef G_OS_WIN32
+  void sighandler (int signum)
+  {
+    std::cerr << "shutting down pan." << std::endl;
+    signal (signum, SIG_DFL);
+    gtk_main_quit ();
+  }
+#endif // G_OS_WIN32
+
+  void register_shutdown_signals ()
+  {
+#ifndef G_OS_WIN32
+    signal (SIGHUP, sighandler);
+    signal (SIGINT, sighandler);
+    signal (SIGTERM, sighandler);
+#endif // G_OS_WIN32
+  }
+
   void destroy_cb (GtkWidget*w, gpointer user_data)
   {
     gtk_main_quit ();
@@ -108,6 +127,7 @@ namespace
         g_signal_connect (w, "destroy", G_CALLBACK(add_grouplist_task), foo);
       }
 
+      register_shutdown_signals ();
       gtk_main ();
       g_signal_handler_disconnect (window, delete_cb_id);
     }
@@ -225,9 +245,11 @@ main (int argc, char *argv[])
         gtk_widget_show_all (w);
         g_signal_connect (w, "destroy", G_CALLBACK(destroy_cb), 0);
         g_signal_connect (G_OBJECT(w), "delete-event", G_CALLBACK(delete_event_cb), 0);
+        register_shutdown_signals ();
         gtk_main (); // FIXME: never exits when task pane user hits 'close'
       } else {
         GMainLoop * main_loop = g_main_loop_new (NULL, false);
+        register_shutdown_signals ();
         g_main_loop_run (main_loop); // FIXME g_loop_quit when queue.empty()
         g_main_loop_unref (main_loop);
       }
