@@ -44,6 +44,7 @@ extern "C" {
 #include "post-ui.h"
 #include "post.ui.h"
 #include "profiles-dialog.h"
+#include "url.h"
 
 #ifdef HAVE_GTKSPELL
 #define DEFAULT_SPELLCHECK_FLAG true
@@ -218,7 +219,7 @@ namespace
   void do_edit     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->spawn_editor (); }
   void do_editors  (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->manage_editors (); }
   void do_profiles (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->manage_profiles (); }
-  void do_post     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->post_now (); }
+  void do_send     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->send_now (); }
   void do_save     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->save_draft (); }
   void do_open     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->open_draft (); }
   void do_close    (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->close_window (); }
@@ -240,8 +241,8 @@ namespace
     { "charsets-menu", NULL, N_("Character Encoding") },
     { "editors-menu", NULL, N_("Set Editor") },
     { "post-toolbar", NULL, "post" },
-    { "post-article", GTK_STOCK_EXECUTE, N_("_Post Article"), "<control>Return", N_("Post this Article Now"), G_CALLBACK(do_post) },
-    { "save-draft", GTK_STOCK_SAVE, N_("_Save Draft"), "<control>s", N_("Save as a Draft for Future Posting"), G_CALLBACK(do_save) },
+    { "post-article", GTK_STOCK_EXECUTE, N_("_Send Article"), "<control>Return", N_("Send Article Now"), G_CALLBACK(do_send) },
+    { "save-draft", GTK_STOCK_SAVE, N_("Sa_ve Draft"), "<control>s", N_("Save as a Draft for Future Posting"), G_CALLBACK(do_save) },
     { "open-draft", GTK_STOCK_OPEN, N_("_Open Draft..."), "<control>o", N_("Open an Article Draft"), G_CALLBACK(do_open) },
     { "rewrap", GTK_STOCK_JUSTIFY_FILL, N_("Wrap _Now"), NULL, N_("Wrap the Article Body to 80 Columns"), G_CALLBACK(do_wrap) },
     { "close", GTK_STOCK_CLOSE, N_("_Close"), NULL, N_("Close Window"), G_CALLBACK(do_close) },
@@ -589,20 +590,6 @@ namespace
     static_cast<PostUI*>(user_data)->close_window ();
     return true; // don't invoke the default handler that destroys the widget
   }
-
-  void message_dialog_set_text (GtkMessageDialog * dialog,
-                                const char * primary,
-                                const char * secondary)
-  {
-#if GTK_CHECK_VERSION(2,6,0)
-    gtk_message_dialog_set_markup (dialog, primary);
-    gtk_message_dialog_format_secondary_text (dialog, "%s", secondary);
-#else
-    char * pch = g_strdup_printf ("<b>%s</b>\n\n%s", primary, secondary);
-    gtk_message_dialog_set_markup (dialog, pch);
-    g_free (pch);
-#endif
-  }
 }
 
 void
@@ -618,9 +605,9 @@ PostUI :: close_window ()
       GTK_WINDOW(_root),
       GTK_DIALOG_DESTROY_WITH_PARENT,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, NULL);
-    message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
+    HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
       _("Your changes will be lost!"),
-      _("Are you sure you want to close this window?"));
+      _("Close this window and lose your changes?"));
     gtk_dialog_add_buttons (GTK_DIALOG(d),
                             GTK_STOCK_GO_BACK, GTK_RESPONSE_NO,
                             GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -656,7 +643,7 @@ PostUI :: check_message (const Quark& server, GMimeMessage * msg)
   GtkWidget * d = gtk_message_dialog_new (GTK_WINDOW(_root),
                                           GTK_DIALOG_DESTROY_WITH_PARENT,
                                           type, GTK_BUTTONS_NONE, NULL);
-  message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
+  HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
                            _("There were problems with this post."),
                            s.c_str());
   gtk_dialog_add_button (GTK_DIALOG(d), _("Go Back"), GTK_RESPONSE_CANCEL);
@@ -698,23 +685,6 @@ namespace
   }
 }
 
-void
-PostUI :: on_progress_finished (Progress&, int status)
-{
-  _post_task->remove_listener (this);
-  gtk_widget_destroy (_post_dialog);
-  if (status == OK)
-  {
-    _unchanged_body = get_text (_body_buf);
-    close_window ();
-  }
-  else
-  {
-    //if (status == FAIL)
-    std::cerr << LINE_ID << " status " << status << std::endl;
-  }
-}
-
 namespace
 {
 #if !GTK_CHECK_VERSION(2,6,0)
@@ -728,18 +698,117 @@ namespace
     return text;
   }
 #endif
+}
 
-  void clear_pointer_on_destroy_signal (GtkWidget*& w)
+void
+PostUI :: send_now ()
+{
+  GMimeMessage * message (new_message_from_ui (POSTING));
+  if (!maybe_post_message (message))
+    g_object_unref (G_OBJECT(message));
+}
+
+void
+PostUI :: done_sending_message (GMimeMessage * message, bool ok)
+{
+  if (ok) {
+    _unchanged_body = get_text (_body_buf);
+    close_window ();
+  }
+
+  g_object_unref (G_OBJECT(message));
+}
+
+namespace
+{
+  // http://www.rfc-editor.org/rfc/rfc1738.txt
+  std::string encode (const StringView& in)
   {
-    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
+    static std::set<char> keep;
+    if (keep.empty()) {
+      for (int i='0'; i<='9'; ++i) keep.insert(i); // always okay
+      for (int i='a'; i<='z'; ++i) keep.insert(i); // always okay
+      for (int i='A'; i<='Z'; ++i) keep.insert(i); // always okay
+      for (char *pch="$-_.+!*'(),"; *pch; ++pch) keep.insert(*pch); // okay in the right context
+      for (char *pch="$&+,/:;=?@"; *pch; ++pch) keep.erase(*pch); // reserved
+      for (char *pch="()"; *pch; ++pch) keep.erase(*pch); // gives thunderbird problems?
+    }
+
+    std::string out;
+    foreach_const (StringView, in, pch) {
+      if (keep.count(*pch))
+        out += *pch;
+      else {
+        char buf[8];
+        g_snprintf (buf, sizeof(buf), "%%%02x", (int)*pch);
+        out += buf;
+      }
+    }
+    return out;
   }
 }
 
 void
-PostUI :: post_now ()
+PostUI :: maybe_mail_message (GMimeMessage * message)
 {
-  GMimeMessage * message = new_message_from_ui (POSTING);
+  std::string url, to, groups;
+  gboolean unused;
+  char * headers (g_mime_message_get_headers (message));
+  char * body (g_mime_message_get_body (message, true, &unused));
+  StringView key, val, v(headers);
+  v.trim ();
+  while (v.pop_token (val, '\n') && val.pop_token(key,':')) {
+    key.trim ();
+    val.eat_chars (1);
+    val.trim ();
+    std::string key_enc (encode (key));
+    std::string val_enc (encode (val));
+    if (key == "To")
+      to = val_enc;
+    else if (key == "Newsgroups")
+      groups = val;
+    else if (key!="User-Agent" && key!="Mime-Version" && key!="Content-Type") {
+      url += '&';
+      url += key_enc;
+      url += '=';
+      url += val_enc;
+    }
+  }
 
+  if (!groups.empty()) {
+    char * pch = g_strdup_printf ("[This mail was also posted to %s.]\n\n%s", groups.c_str(), body);
+    g_free (body);
+    body = pch;
+  }
+
+  url += std::string("&body=") + encode(body);
+  url[0] = '?';
+  url = std::string("mailto:") + to + url;
+
+  if (!to.empty())
+    URL :: open (_prefs, url.c_str(), URL::MAIL);
+  done_sending_message (message, true);
+
+  g_free (body);
+  g_free (headers);
+}
+
+void
+PostUI :: on_progress_finished (Progress&, int status) // posting finished
+{
+  _post_task->remove_listener (this);
+  gtk_widget_destroy (_post_dialog);
+
+  GMimeMessage * message (_post_task->get_message ());
+  if (status != OK) // error posting.. stop.
+    done_sending_message (message, false);
+  else
+    maybe_mail_message (message);
+}
+
+bool
+PostUI :: maybe_post_message (GMimeMessage * message)
+{
   /**
   ***  Find the server to use
   **/
@@ -765,15 +834,24 @@ PostUI :: post_now ()
       _("No posting server is set for this posting profile.\nPlease edit the profile via Edit|Manage Posting Profiles."));
     gtk_dialog_run (GTK_DIALOG(d));
     gtk_widget_destroy (d);
-    return;
+    return false;
   }
 
   /**
   ***  Make sure the message is OK...
   **/
-  if (!check_message (server, message)) {
-    g_object_unref (G_OBJECT(message));
-    return;
+
+  if (!check_message (server, message))
+    return false;
+
+  /**
+  *** If this is email only, skip the rest of the posting...
+  *** we only stayed this long to get check_message()
+  **/
+  const StringView groups (g_mime_message_get_header (message, "Newsgroups"));
+  if (groups.empty()) {
+    maybe_mail_message (message);
+    return true;
   }
 
   /**
@@ -785,17 +863,15 @@ PostUI :: post_now ()
                                             GTK_DIALOG_DESTROY_WITH_PARENT,
                                             GTK_MESSAGE_QUESTION,
                                             GTK_BUTTONS_NONE, NULL);
-    message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
+    HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
                              _("Pan is Offline."),
                              _("Go online to post the article?"));
     gtk_dialog_add_button (GTK_DIALOG(d), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
     gtk_dialog_add_action_widget (GTK_DIALOG(d), new_go_online_button(), GTK_RESPONSE_OK);
     const int response = gtk_dialog_run (GTK_DIALOG(d));
     gtk_widget_destroy (d);
-    if (response !=  GTK_RESPONSE_OK) {
-      g_object_unref (G_OBJECT(message));
-      return;
-    }
+    if (response !=  GTK_RESPONSE_OK)
+      return false;
     _queue.set_online (true);
   }
 
@@ -810,8 +886,6 @@ PostUI :: post_now ()
   char buf[512];
   g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Posting..."));
   GtkWidget * w = GTK_WIDGET (g_object_new (GTK_TYPE_LABEL, "use-markup", TRUE, "label", buf, NULL));
-  //GtkWidget * w = gtk_label_new (0);
-  //gtk_label_set_markup (GTK_LABEL(w), buf);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d)->vbox), w, false, false, PAD_SMALL);
   w = gtk_progress_bar_new ();
   gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR(w), 0.05);
@@ -819,13 +893,12 @@ PostUI :: post_now ()
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d)->vbox), w, false, false, PAD_SMALL);
   g_object_set_data_full (G_OBJECT(d), "progressbar-timeout-tag", GUINT_TO_POINTER(tag), remove_progress_tag);
   _post_dialog = d;
-  clear_pointer_on_destroy_signal (_post_dialog);
+  g_signal_connect (_post_dialog, "destroy", G_CALLBACK(gtk_widget_destroyed), &_post_dialog);
   gtk_widget_show_all (d);
   _post_task = new TaskPost (server, message);
   _post_task->add_listener (this);
   
   _queue.add_task (_post_task);
-  g_object_unref (message);
 
   /**
   ***  Maybe remember the charsets.
@@ -839,8 +912,14 @@ PostUI :: post_now ()
         _group_prefs.set_string (groupname, "character-encoding", _charset);
     }
   }
+
+  return true;
 }
 
+/***
+****
+***/
+
 void
 PostUI :: spawn_editor ()
 {
@@ -1035,6 +1114,7 @@ namespace
   {
     return (name == "Subject")
         || (name == "Newsgroups")
+        || (name == "To")
         || (name == "From");
   }
 
@@ -1073,9 +1153,15 @@ PostUI :: new_message_from_ui (Mode mode)
   const char * cpch (gtk_entry_get_text (GTK_ENTRY(_subject_entry)));
   g_mime_message_set_subject (msg, cpch);
 
+  // headers from the ui: To
+  const StringView to (gtk_entry_get_text (GTK_ENTRY(_to_entry)));
+  if (!to.empty())
+    g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.str);
+
   // headers from the ui: Newsgroups
-  cpch = gtk_entry_get_text (GTK_ENTRY(_groups_entry));
-  g_mime_message_set_header (msg, "Newsgroups", cpch);
+  const StringView groups (gtk_entry_get_text (GTK_ENTRY(_groups_entry)));
+  if (!groups.empty())
+    g_mime_message_set_header (msg, "Newsgroups", groups.str);
 
   // add the 'hidden headers'
   foreach_const (str2str_t, _hidden_headers, it)
@@ -1130,6 +1216,7 @@ PostUI :: new_message_from_ui (Mode mode)
   g_free (pch);
   g_mime_part_set_content_type (part, type); // part owns type now. type isn't refcounted.
   g_mime_part_set_content_object (part, content_object);
+  g_mime_part_set_encoding (part, GMIME_PART_ENCODING_8BIT);
   g_object_unref (content_object);
   g_mime_message_set_mime_part (msg, GMIME_OBJECT(part));
   g_object_unref (part);
@@ -1352,9 +1439,9 @@ namespace
      * which could result in a blank message in the composer window. */
     if (sig!=NULL)
     {
-      char * sig_utf8 = pan :: content_to_utf8 (sig, -1);
+      const std::string s (content_to_utf8 (sig));
       g_free (sig);
-      sig = sig_utf8;
+      sig = g_strndup (s.c_str(), s.size());
     }
     else
     {
@@ -1442,12 +1529,7 @@ PostUI :: apply_profile_to_body ()
   if (!attribution.empty())
   {
     // scrub the attribution for UTF8 cleanness
-    char * pch = content_to_utf8 (attribution.c_str(), attribution.size(), 0, 0);
-    if (pch)
-      attribution = pch;
-    else
-      attribution.clear ();
-    g_free (pch);
+    attribution = header_to_utf8 (attribution);
  
     std::string::size_type pos = body.find (old_attribution);
     if (!old_attribution.empty() && (pos != std::string::npos))
@@ -1588,11 +1670,15 @@ PostUI :: set_message (GMimeMessage * message)
     g_object_unref (G_OBJECT(_message));
   _message = message;
 
-  // update subject, newsgroups fields
-  const char * pch = g_mime_message_get_subject (message);
-  gtk_entry_set_text (GTK_ENTRY(_subject_entry), pch ? pch : "");
-  pch = g_mime_message_get_header (message, "Newsgroups");
-  gtk_entry_set_text (GTK_ENTRY(_groups_entry), pch ? pch : "");
+  // update subject, newsgroups, to fields
+  const char * cpch = g_mime_message_get_subject (message);
+  gtk_entry_set_text (GTK_ENTRY(_subject_entry), cpch ? cpch : "");
+  cpch = g_mime_message_get_header (message, "Newsgroups");
+  gtk_entry_set_text (GTK_ENTRY(_groups_entry), cpch ? cpch : "");
+  const InternetAddressList * addresses = g_mime_message_get_recipients (message, GMIME_RECIPIENT_TYPE_TO);
+  char * pch  = internet_address_list_to_string (addresses, true);
+  gtk_entry_set_text (GTK_ENTRY(_to_entry), pch ? pch : "");
+  g_free (pch);
 
   // update 'other headers'
   SetMessageForeachHeaderData data;
@@ -1674,8 +1760,7 @@ PostUI :: PostUI (Data         & data,
                   Profiles     & profiles,
                   GMimeMessage * message,
                   Prefs        & prefs,
-                  GroupPrefs   & group_prefs,
-                  GtkWindow    * parent) :
+                  GroupPrefs   & group_prefs):
   _data (data),
   _queue (queue),
   _gs (gs),
@@ -1686,6 +1771,7 @@ PostUI :: PostUI (Data         & data,
   _from_combo (0),
   _subject_entry (0),
   _groups_entry (0),
+  _to_entry (0),
   _body_view (0),
   _body_buf (0),
   _message (0),
@@ -1743,6 +1829,12 @@ PostUI :: PostUI (Data         & data,
   gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
   gtk_table_attach (GTK_TABLE(t), w, 3, 4, row, row+1, fill, nofill, 0, 0);
   ++row;
+  g_snprintf (buf, sizeof(buf), "<b>%s:</b>", _("_Mail To"));
+  l = HIG :: workarea_add_label (t, row, buf);
+  w = _to_entry = gtk_entry_new ();
+  gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
+  gtk_table_attach (GTK_TABLE(t), w, 3, 4, row, row+1, fill, nofill, 0, 0);
+  ++row;
   GtkWidget * frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
   GtkWidget * v = gtk_text_view_new ();
@@ -1766,12 +1858,6 @@ PostUI :: PostUI (Data         & data,
   g_signal_connect (_body_buf, "insert-text", G_CALLBACK(text_was_inserted_cb), &_tm);
   gtk_box_pack_start (GTK_BOX(vbox), w, true, true, 0);
 
-  // wire ourselves up with the parent
-  if (parent != 0) {
-    gtk_window_set_transient_for (GTK_WINDOW(_root), parent);
-    gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ON_PARENT);
-  }
-
   // ensure that we've got at least one profile to choose from...
   gtk_widget_show_all (vbox);
   if (!_profiles.has_profiles()) {
@@ -1787,6 +1873,8 @@ PostUI :: PostUI (Data         & data,
     gtk_widget_destroy (d.root());
   }
 
+  gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ALWAYS);
+
   set_message (message);
 
   _unchanged_body = get_text (_body_buf);
@@ -1799,17 +1887,12 @@ PostUI :: PostUI (Data         & data,
       grab = _subject_entry;
   }
   if (!grab) {
-    const char * cpch (gtk_entry_get_text (GTK_ENTRY(_groups_entry)));
-    if (!cpch || !*cpch)
+    const StringView one (gtk_entry_get_text (GTK_ENTRY(_groups_entry)));
+    const StringView two (gtk_entry_get_text (GTK_ENTRY(_to_entry)));
+    if (one.empty() && two.empty())
       grab = _groups_entry;
   }
   if (!grab)
     grab = _body_view;
   gtk_widget_grab_focus (grab);
-
-  clear_pointer_on_destroy_signal (_root);
-  clear_pointer_on_destroy_signal (_from_combo);
-  clear_pointer_on_destroy_signal (_subject_entry);
-  clear_pointer_on_destroy_signal (_groups_entry);
-  clear_pointer_on_destroy_signal (_body_view);
 }
