@@ -30,6 +30,7 @@
 namespace pan
 {
   class Profiles;
+  class TaskPost;
 
   /**
    * Dialog for posting new messages Pan's GTK GUI.
@@ -39,7 +40,7 @@ namespace pan
   {
     public:
       PostUI (Data&, Queue&, GroupServer&, Profiles&, GMimeMessage* message,
-              Prefs&, GroupPrefs&, GtkWindow * parent=0);
+              Prefs&, GroupPrefs&);
       ~PostUI ();
 
     public:
@@ -54,9 +55,14 @@ namespace pan
       void apply_profile ();
       void save_draft ();
       void open_draft ();
-      void post_now ();
+      void send_now ();
       void close_window ();
 
+    private:
+      void done_sending_message (GMimeMessage*, bool);
+      void maybe_mail_message (GMimeMessage*);
+      bool maybe_post_message (GMimeMessage*);
+
     private:
       void add_editors_to_menu ();
       void update_widgetry ();
@@ -78,6 +84,7 @@ namespace pan
       GtkWidget * _from_combo;
       GtkWidget * _subject_entry;
       GtkWidget * _groups_entry;
+      GtkWidget * _to_entry;
       GtkWidget * _body_view;
       GtkTextBuffer * _body_buf;
       GtkTextBuffer * _headers_buf;
@@ -89,7 +96,7 @@ namespace pan
       GtkActionGroup * _agroup;
       std::string _signature;
       GtkWidget * _post_dialog;
-      Task * _post_task;
+      TaskPost * _post_task;
       typedef std::map<std::string, std::string> str2str_t;
       str2str_t _hidden_headers;
       str2str_t _profile_headers;
