@@ -27,7 +27,9 @@ extern "C" {
 #include <pan/icons/pan-pixbufs.h>
 #include "hig.h"
 #include "pad.h"
+#include "pan-file-entry.h"
 #include "prefs-ui.h"
+#include "url.h"
 
 using namespace pan;
 
@@ -98,6 +100,102 @@ namespace
     return r;
   }
 
+  void set_prefs_string_from_editable (GtkEditable * editable, gpointer prefs_gpointer)
+  {
+    Prefs * prefs (static_cast<Prefs*>(prefs_gpointer));
+    const char * key = (const char*) g_object_get_data (G_OBJECT(editable), PREFS_KEY);
+    char * val = gtk_editable_get_chars (editable, 0, -1);
+    prefs->set_string (key, val);
+    g_free (val);
+  }
+
+  void maybe_make_widget_visible (GtkComboBox * c, gpointer user_data)
+  {
+    GtkWidget * w (GTK_WIDGET(user_data));
+    GtkWidget * c_parent (gtk_widget_get_parent (GTK_WIDGET(c)));
+    GtkWidget * w_parent (gtk_widget_get_parent (GTK_WIDGET(w)));
+    GtkTreeModel * model = gtk_combo_box_get_model (c);
+    const int n_rows (gtk_tree_model_iter_n_children (model, NULL));
+    const bool do_show (gtk_combo_box_get_active(c) == (n_rows-1));
+    if (do_show && !w_parent) // add it
+    {
+      gtk_box_pack_start (GTK_BOX(c_parent), w, true, true, 0);
+      gtk_widget_show (w);
+      g_object_unref (G_OBJECT(w));
+    }
+    else if (!do_show && w_parent) // remove it
+    {
+      g_object_ref (G_OBJECT(w));
+      gtk_container_remove (GTK_CONTAINER(w_parent), w);
+    }
+  }
+
+  void set_prefs_string_from_combobox (GtkComboBox * c, gpointer user_data)
+  {
+    Prefs * prefs (static_cast<Prefs*>(user_data));
+    const char * key = (const char*) g_object_get_data (G_OBJECT(c), PREFS_KEY);
+    const int column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(c), "column"));
+    const int row (gtk_combo_box_get_active (c));
+    GtkTreeModel * m = gtk_combo_box_get_model (c);
+    GtkTreeIter i;
+    if (gtk_tree_model_iter_nth_child (m, &i, 0, row)) {
+      char * val (0);
+      gtk_tree_model_get (m, &i, column, &val, -1);
+      prefs->set_string (key, val);
+      g_free (val);
+    }
+  }
+
+  GtkWidget* url_handler_new (Prefs& prefs,
+                              const char * mode_key,
+                              const char * mode_fallback,
+                              const char * custom_key,
+                              const char * custom_fallback,
+                              GtkWidget *& setme_mnemonic_target)
+  {
+    // build the combo box...
+    const std::string mode (prefs.get_string (mode_key, mode_fallback));
+    GtkListStore * store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+    const char* strings[5][2] = { { N_("Use GNOME Preferences"), "gnome" },
+                                  { N_("Use KDE Preferences"), "kde" },
+                                  { N_("Use OS/X Preferences"), "mac" },
+                                  { N_("Use Windows Preferences"), "windows" },
+                                  { N_("Custom Command:"), "custom" } };
+    int sel_index (0);
+    for (size_t i=0; i<G_N_ELEMENTS(strings); ++i) {
+      GtkTreeIter iter;
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter, 0, _(strings[i][0]), 1, strings[i][1], -1);
+      if (mode == strings[i][1])
+        sel_index = i;
+    }
+    GtkWidget * c = gtk_combo_box_new_with_model (GTK_TREE_MODEL(store));
+    GtkCellRenderer * renderer (gtk_cell_renderer_text_new ());
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (c), renderer, true);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, NULL);
+    gtk_combo_box_set_active (GTK_COMBO_BOX(c), sel_index);
+    g_object_set_data_full (G_OBJECT(c), PREFS_KEY, g_strdup(mode_key), g_free);
+    g_object_set_data (G_OBJECT(c), "column", GINT_TO_POINTER(1));
+    g_signal_connect (c, "changed", G_CALLBACK(set_prefs_string_from_combobox), &prefs);
+
+    // build the custom entry...
+    GtkWidget * e = gtk_entry_new ();
+    const std::string custom (prefs.get_string (custom_key, custom_fallback));
+    gtk_entry_set_text (GTK_ENTRY(e), custom.c_str());
+    g_object_set_data_full (G_OBJECT(e), PREFS_KEY, g_strdup(custom_key), g_free);
+    g_signal_connect (e, "changed", G_CALLBACK(set_prefs_string_from_editable), &prefs);
+
+    // tie them together...
+    g_signal_connect (c, "changed", G_CALLBACK(maybe_make_widget_visible), e);
+    GtkWidget * h = gtk_hbox_new (false, PAD);
+    gtk_box_pack_start (GTK_BOX(h), c, true, true, 0);
+    gtk_box_pack_start (GTK_BOX(h), e, true, true, 0);
+
+    maybe_make_widget_visible (GTK_COMBO_BOX(c), e);
+    setme_mnemonic_target = c;
+    return h;
+  }
+
   void font_set_cb (GtkFontButton* b, gpointer prefs_gpointer)
   {
     const char * key = (const char*) g_object_get_data (G_OBJECT(b), PREFS_KEY);
@@ -446,6 +544,18 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   HIG :: workarea_finish (t, &row);
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, gtk_label_new_with_mnemonic(_("_Colors")));
 
+  row = 0;
+  t = HIG :: workarea_create ();
+  HIG :: workarea_add_section_title (t, &row, _("Preferred Applications"));
+    HIG :: workarea_add_section_spacer (t, row, 2);
+    w = url_handler_new (prefs, "browser-mode", URL::get_environment(),
+                                "custom-browser", "firefox", b);
+    HIG :: workarea_add_row (t, &row, _("_Web Browser:"), w, b);
+    w = url_handler_new (prefs, "mailer-mode", URL::get_environment(),
+                                "custom-mailer", "thunderbird", b);
+    HIG :: workarea_add_row (t, &row, _("_Mail Reader:"), w, b);
+  HIG :: workarea_finish (t, &row);
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, gtk_label_new_with_mnemonic(_("A_pplications")));
 
   gtk_widget_show_all (notebook);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook, true, true, 0);
