@@ -231,7 +231,7 @@ SaveDialog :: SaveDialog (Prefs                       & prefs,
     HIG :: workarea_add_wide_control (t, &row, w);
 
     // bottom of queue
-    w = _position_bottom_radio = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON(w), _("Add to the _back the queue"));
+    w = _position_bottom_radio = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON(w), _("Add to the _back of the queue"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), false);
     HIG :: workarea_add_wide_control (t, &row, w);
 
