@@ -373,9 +373,8 @@ namespace
       str = g_markup_printf_escaped ("%s\n<small>%s</small>", description.c_str(), status);
     else
       str = g_markup_printf_escaped ("%s\n<small>%s</small>", description.c_str(), path);
-    char * str_utf8 = pan::clean_utf8 (str, -1);
-    g_object_set(rend, "markup", str_utf8, "xpad", 10, "ypad", 5, NULL);
-    g_free (str_utf8);
+    const std::string str_utf8 = clean_utf8 (str);
+    g_object_set(rend, "markup", str_utf8.c_str(), "xpad", 10, "ypad", 5, NULL);
     g_free (str);
   }
 }
@@ -398,12 +397,6 @@ TaskPane :: ~TaskPane ()
   g_source_remove (_update_timeout_tag);
 }
 
-namespace {
-  void clear_pointer_on_destroy_signal (GtkWidget*& w) {
-    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
-  }
-}
-
 TaskPane :: TaskPane (Queue& queue, Prefs& prefs): _queue(queue)
 {
   _root = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -505,8 +498,4 @@ TaskPane :: TaskPane (Queue& queue, Prefs& prefs): _queue(queue)
   gtk_widget_show_all (vbox);
   prefs.set_window ("tasks-window", GTK_WINDOW(_root), 200, 200, 550, 600);
   gtk_window_set_resizable (GTK_WINDOW(_root), true);
-
-  clear_pointer_on_destroy_signal (_root);
-  clear_pointer_on_destroy_signal (_view);
-  clear_pointer_on_destroy_signal (_status_label);
 }
