@@ -291,8 +291,8 @@ if (line.strstr ("eagifg$7fo$1@sunflower.man.poznan.pl") != 0) std::cerr << LINE
   h = std::max (h, number);
 
   const char * fallback_charset = NULL; // FIXME
-  char * subject_utf8 = header_to_utf8 (subj, fallback_charset);
-  char * author_utf8 = header_to_utf8 (author, fallback_charset);
+  const std::string subject_utf8 = header_to_utf8 (subj, fallback_charset);
+  const std::string author_utf8 = header_to_utf8 (author, fallback_charset);
 
   ++_parts_so_far;
 
@@ -322,8 +322,6 @@ if (line.strstr ("eagifg$7fo$1@sunflower.man.poznan.pl") != 0) std::cerr << LINE
   }
 
   // cleanup
-  g_free (author_utf8);
-  g_free (subject_utf8);
   g_free (buf);
 }
 
